@@ -16,15 +16,10 @@ QNetworkReply* SyncNetworkManagerProxy::get(const QUrl &url) {
 QNetworkReply* SyncNetworkManagerProxy::get(const QNetworkRequest &request) {
     QNetworkReply *reply = p_manager->get(request);
 
-#ifdef PHANTOMPY_QT4
-    connect(reply, SIGNAL(finished()), this, SLOT(finished()));
-    m_eventLoop.exec();
-#else
     QMetaObject::Connection connection = connect(reply, &QNetworkReply::finished,
                                                  this, &SyncNetworkManagerProxy::finished);
     m_eventLoop.exec();
     QObject::disconnect(connection);
-#endif
 
     return reply;
 }
